<?php

/**
 * Deals with various available shortcodes
 *
 * @link       https://www.pedigreeonline.com
 * @since      1.0.0
 *
 * @package    Pedigree_Online
 * @subpackage Pedigree_Online/shortcodes
 */

class Pedigree_Online_Functions {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	* "Cache" the options once we've retrived them so we don't query the DB over and over
	*
	* @since 1.0.0
	*/
	private static $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	* Used to parse out the subdomain (animal), uuid, report and any params (e.g. gens=) from the URL
	*
	* @since 1.0.0
	* @returns Object ( animal, uuid, report, gens )
	*
	*/
	public function parse_url( $url ) {

	  $url = html_entity_decode( $url ); // Removes &amp;

	  $parts = parse_url($url);

	  $return = (object) array( 'animal' => null, 'uuid' => null, 'report' => null, 'gens' => null );

	  // If there's no host, or there is but it's not a pedigreeonline.com domain...
	  if ( ! isset( $parts['host'] ) || ! stristr( $parts['host'], Pedigree_Online::ROOT_DOMAIN ) ) {

	    return $return;

	  }

	  $return->animal = str_ireplace('.' . Pedigree_Online::ROOT_DOMAIN, '', $parts['host']);

	  if ( ! isset( $parts['path'] ) ) {

	    return $return;

	  }

	  // Try to find the slug
	  if ( preg_match("@^.+\-([^\-/]+)/?([^/]*)$@", $parts['path'], $matches) ) {

	      $return->uuid = $matches[1];
	      $return->report = $matches[2];

	  }
	  // Otherwise there might be some special paths for special reports
	  elseif ( preg_match("@^/hypomating/(.+)$@", $parts['path'], $matches) ) {
	      $return->uuid = $matches[1];
	      $return->report = 'saved_hypomating';
	  }

	  // Just a couple re-adjustments
	  if ( $return->report === 'progeny' ) {
	    $return->report = 'offspring';
	  }

    // See if we have a "gens" parameter
	  if ( isset( $parts['query'] )
	       && ( parse_str( $parts['query'], $query ) || $query )
	       && isset( $query['gens'] )
	       && is_numeric( $query['gens']) ) {

	       $return->gens = $query['gens'];

	  }

	  return $return;

	}


	public function call_api( $animal, $uuid, $report = 'pedigree', $params = array()) {

    // First make sure we have a token
    $api_token = self::get_option( 'api_token' );

    if ( empty( $api_token )) {

      return (object) array('result' => false, 'message' => __("There is no API Token defined in your settings. ".
        "Please visit the Wordpress Settings area to view instructions on how to obtain an API Token for this plugin."));

    }

    // First check our cache to see if this has been seen recently
    $cache_key = Pedigree_Online::OPTION_PREFIX . $uuid . '_' . $report . ( isset($params['gens']) ? '_'.$params['gens'] : '' );

    if ( false === ($response = get_transient( $cache_key )) ) {


      // Build the stub and the full URL.
      $query = http_build_query($params);
      $api_uri = 'https://'.
                  $animal .
                  '.'.Pedigree_Online::ROOT_DOMAIN .
                  '/api/' .
                  $uuid .
                  '/' .
                  $report .
                  '?' .
                  http_build_query($params);

      // @1.1.0 - Interjection for different types of reports with different URL structures
      if ( $report === 'saved_hypomating' ) {
        $api_uri = 'https://'.
                  $animal .
                  '.'.Pedigree_Online::ROOT_DOMAIN .
                  '/api/' .
                  'hypomating' .
                  '/' .
                  $uuid .
                  '?' .
                  http_build_query($params);
      }

      $args = array(
        'headers' => array(
          'Authorization' => $api_token,
        ),
      );

      //print "<pre>".$api_uri."</pre>";

      // Hit it!
      $response = wp_remote_get( $api_uri, $args );

      if ( is_wp_error( $response ) ) {

          return (object) array('result' => false, 'message' => __( $response->get_error_message() ));

      }

      $response = json_decode($response['body']);

      // Cache this if result !== false
      if ( $response->result !== false ) {

        set_transient( $cache_key, $response, Pedigree_Online::CACHE_TIMEOUT);

      }

    }

    return $response;

	}

	/**
	* Returns the individual setting from our array of settings, or everything if no key provided
	*
	* @since 1.0.0
	*/
	public static function get_option( $option_name = null ) {

    // Attempt the static
	  $options = self::$options;

	  if ( empty($options)) {

	    $options = self::$options = get_option( Pedigree_Online::OPTION_PREFIX . 'settings' );

	  }

	  if ( empty( $option_name ) ) {

	    return $options;

	  }

	  if ( ! is_null( $value = $options[$option_name] ) ) {

	    return $value;

	  }

	  return null;

	}

	/**
	*
	* Runs a query to delete any keys that match _transient_* with our prefix
	* Static so we can run it anywhere and it doesn't depend on any instance properties
	*
	* @since 1.0.0
	*
	*/
	public static function clear_transients() {

	  global $wpdb;

	  $wpdb->query("DELETE FROM ". $wpdb->prefix ."options
	    WHERE option_name LIKE '". $wpdb->esc_like( '_transient_' . Pedigree_Online::OPTION_PREFIX )."%'
	    OR option_name LIKE '". $wpdb->esc_like( '_transient_timeout_' . Pedigree_Online::OPTION_PREFIX )."%'");

	}


}
